<?php
/* --------------------------------------------------------------
   LexofficeInvoiceModuleCenterModuleController.inc.php 2020-09-11
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

class LexofficeInvoiceModuleCenterModuleController extends AbstractModuleCenterModuleController
{
    /**
     * Configuration storage
     * @var LexofficeInvoiceConfigurationStorage
     */
    protected $configuration;

    /**
     * lexoffice API Service
     * @var LexofficeInvoiceService
     */
    protected $lexofficeInvoiceService;

    /**
     * lexoffice invoice licence Service
     * @var LexofficeInvoiceLicenseService
     */
    protected $lexofficeInvoiceLicenseService;

    /**
     * @var LexofficeInvoiceLogger $logger
     */
    protected $logger;

    protected function _init()
    {
        $this->pageTitle = $this->languageTextManager->get_text('lexoffice_invoice_page_title');
        $this->configuration = MainFactory::create('LexofficeInvoiceConfigurationStorage');
        $this->lexofficeInvoiceService = MainFactory::create('LexofficeInvoiceService', $this->configuration, $this->languageTextManager);
        $this->lexofficeInvoiceLicenseService = MainFactory::create('LexofficeInvoiceLicenseService', $this->languageTextManager);
        $this->logger = MainFactory::create('LexofficeInvoiceLogger');
    }

    /**
     * Return the default page
     * @return AdminLayoutHttpControllerResponse Layout response
     */
    public function actionDefault()
    {
        $title    = new NonEmptyStringType($this->pageTitle);
        $template = $this->getTemplateFile('GXModules/Agiltech/LexofficeInvoice/Admin/Html/lexoffice_invoice_configuration.html');
        $lxo_messages = [];

        // get license information
        try {
            $isLicenseValid = $this->lexofficeInvoiceLicenseService->isLicenseValid($this->configuration->getLicenseKey());
        } catch (LexofficeInvoiceLicenseException $e) {
            $this->logger->error('Error reading license information: ' . $e);
            // add error message
            array_push($lxo_messages, [
                'type'    => 'danger',
                'content' => $e->getMessage(),
            ]);
        }

        // if having valid access token - load lexoffice profile information
        try {
            if (!empty($this->lexofficeInvoiceService->getAccessToken())) {
                // read lexoffice profile
                $lexofficeProfile = (array)$this->lexofficeInvoiceService->readLexofficeProfile();

                // auto update taxType from profile
                $this->configuration->setTaxType($lexofficeProfile['taxType']);
                $this->configuration->setSmallBusiness($lexofficeProfile['smallBusiness'] ? '1' : '0');
            } else {
                $this->configuration->setTaxType(null);
                $this->configuration->setSmallBusiness('0');
            }
        } catch (LexofficeInvoiceException $e) {
            $this->logger->warning('Cannot read lexoffice profile: ' . $e->getMessage());
            if ($e->getHttpCode() != 401 && $e->getHttpCode() != 403) {
                $this->logger->error('Error reading lexoffice profile: ' . $e);
                // add error message
                array_push($lxo_messages, [
                    'type'    => 'danger',
                    'content' => $e->getMessage(),
                    'details' => $e->getDetails(),
                ]);
            }
        } catch (LexofficeInvoiceAuthException $e) {
            $this->logger->error('Cannot read lexoffice profile until authorization valid: ' . $e);
            // add error message
            array_push($lxo_messages, [
                'type'    => 'danger',
                'content' => $e->getMessage(),
            ]);
        }

        $isLicenseExpired = false;
        // only check if already entered key
        $licenseKey = $this->configuration->getLicenseKey();
        $licenseExpirationDate = $this->configuration->getLicenseExpirationDate();
        if (!empty($licenseKey) && $this->lexofficeInvoiceLicenseService->isTrialLicense($licenseKey)) {
            $isLicenseExpired = $this->lexofficeInvoiceLicenseService->isLicenseExpired($licenseExpirationDate, $licenseKey);
        }

        $config = $this->configuration->getAll();
        $taxInfoTaxfree = gm_get_conf('TAX_INFO_TAX_FREE');

        /**
         * define content dependend lxo messages
         */
        if (($taxInfoTaxfree === 'true' && $config['lexoffice_tax_type'] !== 'vatfree')
            || ($taxInfoTaxfree === 'false' && $config['lexoffice_tax_type'] === 'vatfree')
        ) {
            array_push($lxo_messages, [
                'type'    => 'warning',
                'content' => $this->languageTextManager->get_text('lexoffice_invoice_error_tax'),
            ]);
        }
        if ($config['lexoffice_tax_type'] === 'net') {
            array_push($lxo_messages, [
                'type'    => 'warning',
                'content' => $this->languageTextManager->get_text('lexoffice_invoice_error_net'),
            ]);
        }
        if ($isLicenseExpired === true) {
            array_push($lxo_messages, [
                'type'    => 'warning',
                'content' => $this->languageTextManager->get_text('lexoffice_invoice_error_module_expired'),
            ]);
        }

        $data = MainFactory::create(
            'EditableKeyValueCollection',
            [
                'pageToken'                 => $_SESSION['coo_page_token']->generate_token(),
                'configuration'             => $config,
                'isLicenseValid'            => $isLicenseValid,
                'lexofficeProfile'          => $lexofficeProfile,
                'taxInfoTaxfree'            => $taxInfoTaxfree,
                'lexofficeAuthUrl'          => $this->lexofficeInvoiceService->getAuthorizationUrl(!empty($this->configuration->getLexofficeApiAccessToken())),
                'action_save_configuration' => xtc_href_link(
                    'admin.php',
                    'do=LexofficeInvoiceModuleCenterModule/Save'
                ),
            ]
        );

        // add all lxo messages
        if (!empty($lxo_messages)) {
            $data->setValue('lxo_messages', $lxo_messages);
            $lxo_messages = [];
        }

        return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data);
    }

    /**
     * Save the configuration values
     * @return RedirectHttpControllerResponse Redirect response
     */
    public function actionSave()
    {
        $this->_validatePageToken();

        $configuration = $this->_getPostData('configuration');

        foreach ($configuration as $key => $value) {
            try {
                $this->configuration->set($key, $value);
            } catch (LexofficeInvoiceLicenseException $licenseException) {
                $details = json_decode($licenseException->getDetails()[0], true);
                $action = $details['action'];
                $licenseKey = $details['licenseKey'];
                if ($action === 'activate') {
                    $GLOBALS['messageStack']->add_session(
                        sprintf($this->languageTextManager->get_text('lexoffice_invoice_error_activate_license'), $licenseKey),
                        'warning'
                    );
                }
                if ($action === 'deactivate') {
                    $GLOBALS['messageStack']->add_session(
                        sprintf($this->languageTextManager->get_text('lexoffice_invoice_error_deactivate_license'), $licenseKey),
                        'warning'
                    );
                }
            } catch (LexofficeInvoiceAuthException $e) {
                // add error message
                $GLOBALS['messageStack']->add_session(
                    $e->getMessage(),
                    'error'
                );
            } catch (Exception $exception) {
                $GLOBALS['messageStack']->add_session(
                    $this->languageTextManager->get_text('lexoffice_invoice_error_saving_configuration'),
                    'error'
                );
                $this->logger->error('Cannot save lexoffice configuration: ' . $exception->getMessage());
            }
        }

        $GLOBALS['messageStack']->add_session(
            $this->languageTextManager->get_text('lexoffice_invoice_saved'),
            'info'
        );

        return MainFactory::create(
            'RedirectHttpControllerResponse',
            xtc_href_link('admin.php', 'do=LexofficeInvoiceModuleCenterModule')
        );
    }
}
