<?php

/* --------------------------------------------------------------
   LexofficeInvoiceLicenseService.inc.php 2020-10-13
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

/**
 * Class representing lexoffice invoice license service
 */
class LexofficeInvoiceLicenseService
{
    private const LICENSE_STATUS_DELIVERED = 2;

    /**
     * Defines the trial perios
     */
    const TRIAL_PERIOD = 'P30D';

    /**
     * Agiltech license API URL
     */
    const AGILTECH_LICENSE_API_BASE_URL     = 'https://lxo.agiltech.de/wp-json/lmfwc/v2';
    const AGILTECH_LICENSE_API_BASIC_AUTH   = 'Y2tfNDdlYzAyODIzZGJhOGU4OTY4ZmYyNTUyZGU1ZWQwMGU2OTlmZDJjNDpjc184OTYzOGI4Yzc3MzJhNDk1YmM0NjU5MGExMzJjMDc1ZTJiZjllYmY4';

    /**
     * @var LanguageTextManager $languageTextManager
     */
    protected $languageTextManager;

    /**
     * @var LexofficeInvoiceLogger $logger
     */
    protected $logger;

    /**
     * Create instance
     *
     */
    public function __construct(
        LanguageTextManager $languageTextManager = null
    ) {
        $this->languageTextManager = $languageTextManager ? $languageTextManager : MainFactory::create('LanguageTextManager', 'module_center_module');
        $this->logger = MainFactory::create('LexofficeInvoiceLogger');
    }

    /**
     * check if a stored licese keey is a trial license
     * @return bool true if license key is  a trial license
     */
    public function isTrialLicense($licenseKey)
    {
        return ($licenseKey && strpos($licenseKey, 'TRIAL-') === 0);
    }

    public function createEncryptedLicenseExpirationDate($licenseKey)
    {
        // when save first time config - set license expiration date to 30 days left
        $date = new DateTime();
        $date->add(new DateInterval(self::TRIAL_PERIOD));
        $licenseExpirationDate = $date->getTimestamp();
        return $this->encryptExpirationDate($licenseExpirationDate, $licenseKey);
    }

    /**
     * @return bool isLicenseExpired
     */
    public function isLicenseExpired($encryptedLexofficeExpirationDate, $licenseKey)
    {
        if (!empty($encryptedLexofficeExpirationDate) && !empty($licenseKey) && $this->isTrialLicense($licenseKey)) {
            $lexofficeExpirationDate = $this->decryptExpirationDate($encryptedLexofficeExpirationDate, $licenseKey);
            $expirationDate = new DateTime();
            $expirationDate->setTimestamp((int)$lexofficeExpirationDate);
        }

        // if no license -> expired
        // if license is not active -> expired
        $licenseValid = false;
        if (!empty($licenseKey)) {
            try {
                $licenseValid = $this->isLicenseValid($licenseKey);
            } catch (LexofficeInvoiceLicenseException $e) {
                // do nothing here
            }
        }

        $datenow = new DateTime();
        // only check expire date for trial license - other licenses check only for license status
        $expired = $licenseKey && $this->isTrialLicense($licenseKey) && $expirationDate && $datenow->getTimestamp() > $expirationDate->getTimestamp();
        if ($licenseValid === false || $expired === true) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * check if license status is active
     * 
     * @param string license key
     * @return bool true if valid license
     */
    public function isLicenseValid(string $licenseKey)
    {
        $licenseValid = false;
        $license = json_decode($this->sendRequest('GET', self::AGILTECH_LICENSE_API_BASE_URL . '/licenses/' . $licenseKey));
        if ($license && $license->data) {
            $timesActivated = $license->data->timesActivated;
            $timesActivatedMax = $license->data->timesActivatedMax;
            $activated = $timesActivated > 0 && $timesActivated <= $timesActivatedMax;
            $statusDelivered = $license->data->status === self::LICENSE_STATUS_DELIVERED ? true : false;
            $licenseValid = $statusDelivered && $activated;
        }

        return $licenseValid;
    }

    /**
     * activate license
     * 
     * @param string license key
     * @return bool true if valid license
     */
    public function activateLicense(string $licenseKey)
    {
        $licenseValid = false;
        $license = json_decode($this->sendRequest('GET', self::AGILTECH_LICENSE_API_BASE_URL . '/licenses/activate/' . $licenseKey));
        if ($license && $license->data) {
            $licenseValid = $license->data->status == self::LICENSE_STATUS_DELIVERED ? true : false;
        }

        return $licenseValid;
    }

    /**
     * deactivate license
     * 
     * @param string license key
     * @return bool true if valid license
     */
    public function deactivateLicense(string $licenseKey)
    {
        $licenseValid = false;
        $license = json_decode($this->sendRequest('GET', self::AGILTECH_LICENSE_API_BASE_URL . '/licenses/deactivate/' . $licenseKey));
        if ($license && $license->data) {
            $licenseValid = $license->data->status == self::LICENSE_STATUS_DELIVERED ? true : false;
        }

        return $licenseValid;
    }

    /**
     * Sending thr API Request
     */
    private function sendRequest($method, $url)
    {
        $curl = curl_init();

        $requestOptions = array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => $this->getRequestHeader(),
        );

        curl_setopt_array($curl, $requestOptions);
        $response = curl_exec($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        return $this->handleLexofficeResponse($response, $httpcode);
    }

    /**
     * Build the required Request Header
     */
    private function getRequestHeader()
    {
        $requestHeader = array(
            "Authorization: Basic " . self::AGILTECH_LICENSE_API_BASIC_AUTH
        );
        return $requestHeader;
    }

    /**
     * Handle API Request Errors
     */
    private function handleLexofficeResponse($response, $httpcode)
    {
        $chkResponse = json_decode($response);
        if ($httpcode == 200 && $chkResponse && $chkResponse->success == true) {
            return $response;
        } else if ($chkResponse->message) {
            $this->logger->error('An error occurded connecting the license service: ' . print_r($response, true));

            $message = $chkResponse->message;
            if ($httpcode == 404) {
                $message = $this->languageTextManager->get_text('lexoffice_invoice_license_api_error_404');
            }

            throw new LexofficeInvoiceLicenseException($message, $httpcode, $chkResponse->data);
        } else {
            $this->logger->error('An error occurded connecting the license service: ' . print_r($response, true));
            throw new LexofficeInvoiceLicenseException('An error occurded connecting the license service', $httpcode);
        }
    }

    private function encryptExpirationDate($decryptedExpirationDate, $key)
    {
        $cipher = 'aes-128-cbc';
        $encryptExpirationDate = null;
        if (!empty($decryptedExpirationDate) && in_array($cipher, openssl_get_cipher_methods())) {
            try {
                $ivlen = openssl_cipher_iv_length($cipher);
                $iv = openssl_random_pseudo_bytes($ivlen);
                $ciphertext = openssl_encrypt($decryptedExpirationDate, $cipher, $key, $options = 0, $iv);
                // append iv to encrypted text to later on use it to decrypt data
                $encryptExpirationDate = $ciphertext . '_:_' . bin2hex($iv);
            } catch (Exception $e) {
                $this->logger->error('Error encrypt lexoffice expiration date: ' . $e);
            }
        }

        return $encryptExpirationDate;
    }

    private function decryptExpirationDate($encryptedExpirationDate, $key)
    {
        $cipher = 'aes-128-cbc';

        // set default to expired date - if cannot decrypt
        $date = new DateTime();
        $date->sub(new DateInterval(self::TRIAL_PERIOD));
        $decryptExpirationDate = $date->getTimestamp();
        if (!empty($encryptedExpirationDate) && in_array($cipher, openssl_get_cipher_methods())) {
            // get encrypted text and iv
            try {
                $parts = explode('_:_', $encryptedExpirationDate);
                $ciphertext = $parts[0];
                $iv = hex2bin($parts[1]);

                $decryptExpirationDate = openssl_decrypt($ciphertext, $cipher, $key, $options = 0, $iv);
            } catch (Exception $e) {
                $this->logger->error('Error decrypt lexoffice expiration date: ' . $e);
            }
        }

        return $decryptExpirationDate;
    }
}
