<?php

/* --------------------------------------------------------------
   LexofficeInvoiceService.inc.php 2020-09-17
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

/**
 * Class representing lexoffice api service
 */
class LexofficeInvoiceService
{
    /**
     * lexoffice API URL
     */
    const LEXOFFICE_AUTH_BASE_URL = 'https://app.lexoffice.de';

    /**
     * lexoffice authorize redirect url
     */
    const LEXOFFICE_AUTH_REDIRECT_URL = '/api/oauth2/authorization_code';

    /**
     * lexoffice API URL
     */
    const LEXOFFICE_API_BASE_URL = 'https://api.lexoffice.io/v1';

    /**
     * lexoffice api client id
     */
    const LEXOFFICE_AUTH_CLIENT_ID = 'e0017410-7f09-402f-8390-6e7a66cdf1d5';

    /**
     * lexoffice Basic Authentication
     */
    const LEXOFFICE_AUTH_BASIC = 'ZTAwMTc0MTAtN2YwOS00MDJmLTgzOTAtNmU3YTY2Y2RmMWQ1OlFqaT1YT2Y/Nm5baWFycnhpUkkhKE44Z04=';

    /**
     * Configuration storage
     * @var LexofficeInvoiceConfigurationStorage
     */
    protected $configurationStorage;

    /**
     * lexoffice invoice licence Service
     * @var LexofficeInvoiceLicenseService
     */
    protected $lexofficeInvoiceLicenseService;

    /**
     * @var LanguageTextManager $languageTextManager
     */
    protected $languageTextManager;

    /**
     * @var LexofficeInvoiceLogger $logger
     */
    protected $logger;

    /**
     * Create instance
     *
     * @param LexofficeInvoiceConfigurationStorage $configurationStorage Configuration storage
     * @param LanguageTextManager $languageTextManager
     */
    public function __construct(
        LexofficeInvoiceConfigurationStorage $configurationStorage,
        LanguageTextManager $languageTextManager = null
    ) {
        $this->configurationStorage = $configurationStorage;
        $this->languageTextManager = $languageTextManager ? $languageTextManager : MainFactory::create('LanguageTextManager', 'module_center_module');
        $this->lexofficeInvoiceLicenseService = MainFactory::create('LexofficeInvoiceLicenseService', $this->languageTextManager);
        $this->logger = MainFactory::create('LexofficeInvoiceLogger');
    }

    /**
     * Build the Authorization URL to request a authorization code for the oauth2 workflow
     * 
     * @param reauthorize defines if this is an initial authorization or an reauthorization
     * @return string authorization url
     */
    public function getAuthorizationUrl(bool $reauthorize = false)
    {
        $authorizationUrl = self::LEXOFFICE_AUTH_BASE_URL . '/api/oauth2/authorize?client_id=' .
            urlencode(self::LEXOFFICE_AUTH_CLIENT_ID) . '&redirect_uri=' .
            self::LEXOFFICE_AUTH_REDIRECT_URL . '&response_type=code';

        if ($reauthorize) {
            $authorizationUrl = $authorizationUrl . '&reauthorize';
        }

        return $authorizationUrl;
    }

    /**
     * Get lexoffice access token from sorage
     * If the stored access tokes expired - try to get a new one from the lexoffice service using existing refresh token
     * Thus the access token will always be refreshed as long as the refresh token is valid and does exist
     */
    public function getAccessToken()
    {
        $refreshToken = $this->configurationStorage->getLexofficeApiRefreshToken();
        $expirationTimestamp = $this->configurationStorage->getLexofficeApiAccessTokenExpirationTimestamp();
        // if refresh token available and access toke expired - get new access token with existing refresh token from api
        if (!empty($refreshToken) && (empty($expirationTimestamp) || $expirationTimestamp <= time())) {

            $tokenInfo = $this->createLexofficeAccessToken($refreshToken);
            $accessToken = $this->configurationStorage->updateAuthData($tokenInfo);

            return $accessToken;
        }

        return $this->configurationStorage->getLexofficeApiAccessToken();
    }

    /**
     * Ceate access and refreshh tokens from authorization code
     * 
     * @return mixed lexoffice access and refresh tokens with expiration information
     * @throws LexofficeInvoiceAuthException if request fails
     */
    public function createLexofficeAuthorizationTokens($authorizationCode)
    {
        $requestHeader = $this->getRequestHeader('AUTH');
        $body = 'grant_type=authorization_code&code=' . $authorizationCode . '&redirect_uri=' . self::LEXOFFICE_AUTH_REDIRECT_URL;
        return json_decode($this->sendRequest('POST', $requestHeader, self::LEXOFFICE_AUTH_BASE_URL . '/api/oauth2/token', $body));
    }

    /**
     * Create new access token from refresh token
     * 
     * @param refreshToken the refresh token of the existing oauth connection
     * @return mixed lexoffice access and refresh tokens with expiration information
     * @throws LexofficeInvoiceAuthException if request fails
     */
    public function createLexofficeAccessToken($refreshToken)
    {
        $requestHeader = $this->getRequestHeader('AUTH');
        $body = 'grant_type=refresh_token&refresh_token=' . $refreshToken;
        return json_decode($this->sendRequest('POST', $requestHeader, self::LEXOFFICE_AUTH_BASE_URL . '/api/oauth2/token', $body));
    }


    /**
     * Revoke an existing oaut connection
     * 
     * @param accessToken the access token of the existing oauth connection
     * @throws LexofficeInvoiceAuthException if request fails
     */
    public function revokeAuthentication()
    {
        $requestHeader = $this->getRequestHeader('AUTH_REVOKE');
        $this->sendRequest('POST', $requestHeader, LexofficeInvoiceService::LEXOFFICE_API_BASE_URL . '/revoke');
    }
    /**
     * Read lexoffice profile
     * 
     * @return mixed lexoffice profile
     * @throws LexofficeInvoiceException if request fails
     */
    public function readLexofficeProfile()
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/profile'));
    }

    /**
     * Read an existing lexoffice contact
     * 
     * @param string lexoffice contact id
     * @return LexofficeContact created lexoffice contact
     * @throws LexofficeInvoiceException if request fails
     */
    public function readLexofficeContact(string $lxoContactId)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/contacts/' . $lxoContactId));
    }

    /**
     * Create a new lexoffice contact
     * 
     * @param LexofficeContact lexoffice contact json definition
     * @return mixed response including lexoffice contact id
     * @throws LexofficeInvoiceException if request fails
     */
    public function createLexofficeContact(LexofficeContact $lxoContact)
    {
        $requestHeader = $this->getRequestHeader('WRITE');
        $body = json_encode($lxoContact);
        return json_decode($this->sendRequest('POST', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/contacts', $body));
    }

    /**
     * Find an existing lexoffice contact using the contacts customer number
     * 
     * @param string lexoffice customer number
     * @return LexofficeContact created lexoffice contact
     * @throws LexofficeInvoiceException if request fails
     */
    public function findLexofficeContactByNumber(int $customerNumber)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/contacts/?number=' . $customerNumber))->content;
    }

    /**
     * Find an existing lexoffice contact using the contacts email address
     * 
     * @param string lexoffice customer email address
     * @return LexofficeContact created lexoffice contact
     * @throws LexofficeInvoiceException if request fails
     */
    public function findLexofficeContactByEmail(string $customerEmail)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/contacts/?email=' . $customerEmail))->content;
    }

    /**
     * Read an existing lexoffice invoice
     * 
     * @param string lexoffice invoice id
     * @return LexofficeInvoice created lexoffice invoice
     * @throws LexofficeInvoiceException if request fails
     */
    public function readLexofficeInvoice(string $lxoInvoiceId)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/invoices/' . $lxoInvoiceId));
    }

    /**
     * Create a new lexoffice invoice
     * 
     * @param LexofficeInvoice lexoffice invoice json definition
     * @return mixed response including lexoffice invoice id
     * @throws LexofficeInvoiceException if request fails
     */
    public function createLexofficeInvoice(LexofficeInvoiceInvoice $lxoInvoice)
    {
        $requestHeader = $this->getRequestHeader('WRITE');
        $body = json_encode($lxoInvoice);
        return json_decode($this->sendRequest('POST', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/invoices?finalize=true', $body));
    }

    /**
     * Render lexoffice invoice Pdf
     * 
     * @param string lexoffice invoice id
     * @return mixed lexoffice invoice file id
     * @throws LexofficeInvoiceException if request fails
     */
    public function renderLexofficeInvoicePdf(string $lxoInvoiceId)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/invoices/' . $lxoInvoiceId . '/document'));
    }

    /**
     * Read an existing lexoffice credit note
     * 
     * @param string lexoffice creditNote id
     * @return LexofficeInvoiceCreditNote created lexoffice credit note
     * @throws LexofficeInvoiceException if request fails
     */
    public function readLexofficeCreditNote(string $lxoCreditNoteId)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/credit-notes/' . $lxoCreditNoteId));
    }

    /**
     * Create a new lexoffice credit note
     * 
     * @param LexofficeInvoiceCreditNote lexoffice creditnote
     * @return mixed response including lexoffice creaditNote id
     * @throws LexofficeInvoiceException if request fails
     */
    public function createLexofficeCreditNote(LexofficeInvoiceCreditNote $lxoCreditNote)
    {
        $requestHeader = $this->getRequestHeader('WRITE');
        $body = json_encode($lxoCreditNote);
        return json_decode($this->sendRequest('POST', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/credit-notes?finalize=true', $body));
    }

    /**
     * Render lexoffice credit note Pdf
     * 
     * @param string lexoffice credit note id
     * @return string lexoffice credit note file id
     * @throws LexofficeInvoiceException if request fails
     */
    public function renderLexofficeCreditNotePdf(string $lxoCreditNoteId)
    {
        $requestHeader = $this->getRequestHeader('READ');
        return json_decode($this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/credit-notes/' . $lxoCreditNoteId . '/document'));
    }

    /**
     * Render lexoffice voucher Pdf
     * 
     * @param string lexoffice file id
     * @return mixed lexoffice voucher pdf
     * @throws LexofficeInvoiceException if request fails
     */
    public function readLexofficeFile(string $lxoFileId)
    {
        $requestHeader = $this->getRequestHeader('DOWNLOAD');
        return $this->sendRequest('GET', $requestHeader, self::LEXOFFICE_API_BASE_URL . '/files/' . $lxoFileId);
    }

    /**
     * Sending thr API Request
     */
    private function sendRequest($method, $requestHeader, $url, $body = null)
    {
        if (
            $method !== 'GET'
            && $this->lexofficeInvoiceLicenseService->isLicenseExpired($this->configurationStorage->getLicenseExpirationDate(), $this->configurationStorage->getLicenseKey()) == true
        ) {
            // if module expired no other requests to lexoffice can be send like read
            throw new LexofficeInvoiceException($this->languageTextManager->get_text('lexoffice_invoice_error_module_expired'), 401);
        }

        $curl = curl_init();

        $requestOptions = array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => $requestHeader,
        );

        if ($method === 'POST' && !empty($body)) {
            $requestOptions[CURLOPT_POSTFIELDS] = $body;
        }

        $this->logger->notice("sendRequest: " . print_r($requestOptions, true));

        curl_setopt_array($curl, $requestOptions);
        $response = curl_exec($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        return $this->handleLexofficeResponse($response, $httpcode);
    }

    /**
     * Build the required Request Header
     */
    private function getRequestHeader(string $requestType)
    {
        switch ($requestType) {
            case 'AUTH':
                $requestHeader = array(
                    'Authorization: Basic ' . self::LEXOFFICE_AUTH_BASIC,
                    'Content-Type: application/x-www-form-urlencoded',
                    'Accept: application/json'
                );
                break;
            case 'AUTH_REVOKE':
                $requestHeader = array(
                    'Authorization: Bearer ' . $this->getAccessToken(),
                    'Content-Type: application/json',
                    'Accept: application/json'
                );
                break;
            case 'READ':
                $requestHeader = array(
                    'Authorization: Bearer ' . $this->getAccessToken(),
                    'Accept: application/json'
                );
                break;
            case 'WRITE':
                $requestHeader = array(
                    'Authorization: Bearer ' . $this->getAccessToken(),
                    'Accept: application/json',
                    'Content-Type: application/json'
                );
                break;
            case 'DOWNLOAD':
                $requestHeader = array(
                    'Authorization: Bearer ' . $this->getAccessToken()
                );
                break;
        }
        return $requestHeader;
    }

    /**
     * Handle API Request Errors
     */
    private function handleLexofficeResponse($response, $httpcode)
    {
        $chkResponse = json_decode($response);
        // $this->logger->notice('lexoffice API response: ' . print_r($response, true));

        if ($httpcode >= 200 && $httpcode < 300) {
            // status 200 - return response
            return $response;
        } else if ($httpcode == 400 && $chkResponse->error && $chkResponse->error === 'invalid_grant') {
            $this->logger->error('A Bad Request was send to the lexoffice API - No valid refresh token: ' . print_r($response, true));
            // no valid tokens - remove invalid data and fore to rerun authorization workflow
            $this->configurationStorage->revokeAuthData();
            // throw error with specific 400 message
            throw new LexofficeInvoiceAuthException($this->languageTextManager->get_text('lexoffice_invoice_api_error_400'), $httpcode);
        } else if ($httpcode == 401) {
            $this->logger->error('Not Authorized to call lexoffice API: ' . print_r($response, true));
            // unauthorized - no valid access token - remove access token to refresh on next call
            $this->configurationStorage->resetAccessToken();
            // throw error with specific 401 message
            throw new LexofficeInvoiceAuthException($this->languageTextManager->get_text('lexoffice_invoice_api_error_401'), $httpcode);
        } else if ($httpcode == 403) {
            $this->logger->error('No access rights for the calles resource of lexoffice API: ' . print_r($response, true));
            // forbidden - the requested resouce cannot be accessed
            // throw error with specific 403 message
            throw new LexofficeInvoiceAuthException($this->languageTextManager->get_text('lexoffice_invoice_api_error_401'), $httpcode);
        } else if ($httpcode == 404) {
            $this->logger->error('Resource not found when calling lexoffice API: ' . $httpcode . print_r($response, true));
            // throw error with specific 404 message
            throw new LexofficeInvoiceException($this->languageTextManager->get_text('lexoffice_invoice_api_error_403'),  $httpcode);
        } else if ($httpcode >= 400 && $httpcode < 500 && $chkResponse->details) {
            // should be only 406 validation errors
            $this->logger->error('An invalid request was send to the lexoffice API: ' . $httpcode . print_r($response, true));
            // throw error including the original error message
            throw new LexofficeInvoiceException($chkResponse->message, $chkResponse->status ? $chkResponse->status : $httpcode, $chkResponse->details);
        } else if ($httpcode >= 400 && $httpcode < 500 && $chkResponse->IssueList) {
            // should be only 406 validation errors
            $this->logger->error('An invalid request was send to the lexoffice API: ' . $httpcode . print_r($response, true));
            // throw error including the original error message
            throw new LexofficeInvoiceException($chkResponse->message, $chkResponse->status ? $chkResponse->status : $httpcode, $chkResponse->IssueList);
        } else if ($httpcode >= 400 && $httpcode < 500  && $chkResponse->message) {
            // should be only 406 validation errors
            $this->logger->error('An invalid request was send to the lexoffice API: ' . $httpcode . print_r($response, true));
            // throw error including the original error message
            throw new LexofficeInvoiceException($chkResponse->message, $chkResponse->status ? $chkResponse->status : $httpcode);
        } else {
            $this->logger->error('An error occurded when calling lexoffice API: ' . $httpcode . print_r($response, true));
            // throw error with technical error message
            throw new LexofficeInvoiceAuthException($this->languageTextManager->get_text('lexoffice_invoice_api_error_unknown'), $httpcode);
        }
    }
}
