<?php
/* --------------------------------------------------------------
   LexofficeInvoiceConfigurationStorage.inc.php 2021-02-17
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

/**
 * Class representing the lexoffice invoice configuration storage
 */
class LexofficeInvoiceConfigurationStorage extends ConfigurationStorage
{
    /**
     * Configuration storage namespace
     */
    const STORAGE_NAMESPACE = 'modules/agiltech/lexoffice_invoice';

    /**
     * Defines the trial perios
     */
    const TRIAL_PERIOD = 'P30D';

    /**
     * Default configuration storage
     */
    protected $defaults;

    /**
     * lexoffice invoice licence Service
     * @var LexofficeInvoiceLicenseService
     */
    protected $lexofficeInvoiceLicenseService;

    /**
     * @var LexofficeInvoiceLogger $logger
     */
    protected $logger;


    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct(self::STORAGE_NAMESPACE);

        $this->lexofficeInvoiceLicenseService = MainFactory::create('LexofficeInvoiceLicenseService');
        $this->lexofficeService = MainFactory::create('LexofficeInvoiceService', $this);
        $this->logger = MainFactory::create('LexofficeInvoiceLogger');

        $this->defaults = [
            'is_enabled' => '0',
            'license_key' => '',
            'license_expiration_date' => '',
            'auth_revoke' => '',
            'lexoffice_api_key' => '',
            'lexoffice_api_authorizationcode' => '',
            'lexoffice_api_access_token' => '',
            'lexoffice_api_access_token_expiration_timestamp' => '',
            'lexoffice_api_refresh_token' => '',
            'lexoffice_api_refresh_token_expiration_timestamp' => '',
            'lexoffice_tax_type' => '',
            'lexoffice_small_business' => '',
            'lexoffice_contact' => '',
            'lexoffice_guest_contact' => '',
            'lexoffice_footer' => '',
        ];
    }

    /**
     * Return a configuration value
     *
     * @param string $key Configuration key
     *
     * @return string Configuration value
     * @throws InvalidArgumentException Invalid key
     */
    public function get($key)
    {
        if (!array_key_exists($key, $this->defaults)) {
            throw new InvalidArgumentException('Invalid key');
        }

        $value = parent::get($key);

        if ($value === false && array_key_exists($key, $this->defaults)) {
            $value = $this->defaults[$key];
        }

        return $value;
    }

    /**
     * Return all configuration key/value pairs
     * @return array Key/Value pairs
     */
    public function getAll()
    {
        $values = parent::get_all();

        foreach ($this->defaults as $key => $defaultValue) {
            if (!array_key_exists($key, $values)) {
                $values[$key] = $defaultValue;
            }
        }

        return $values;
    }

    /**
     * deletes entire namespace - except license expiration date - so on reinstall this will not be reset.
     * Thus user cannot start a new trial period each time installing the module again
     */
    public function delete_all_trial($p_prefix = '')
    {
        $licenseExpirationDate = $this->get('license_expiration_date');
        parent::delete_all();
        parent::set('license_expiration_date', $licenseExpirationDate);
    }


    /**
     * Set a configuration value
     *
     * @param string $key   Configuration key
     * @param string $value Configuration value
     *
     * @return $this Same instance
     * @throws InvalidArgumentException Invalid key
     */
    public function set($key, $value)
    {
        if (!array_key_exists($key, $this->defaults)) {
            throw new InvalidArgumentException('Invalid key');
        }

        $licenseKey = $this->getLicenseKey();
        if ($key === 'license_expiration_date' && empty($value) && !empty($licenseKey) && $this->lexofficeInvoiceLicenseService->isTrialLicense($licenseKey)) {
            // when save first time config - set license expiration date
            $value = $this->lexofficeInvoiceLicenseService->createEncryptedLicenseExpirationDate($licenseKey);
        }

        if ($key === 'license_key') {
            $oldKey = $this->getLicenseKey();
            if ($oldKey !== $value) {
                try {
                    // if old key is no trial - try to dectivate
                    if (!empty($oldKey)) {
                        $this->logger->notice('Deactivate license: ' . $oldKey);
                        $this->lexofficeInvoiceLicenseService->deactivateLicense($oldKey);
                    }
                } catch (LexofficeInvoiceLicenseException $e) {
                    $details = '{"action":"deactivate","licenseKey":"' . $oldKey . '"}';
                    throw new LexofficeInvoiceLicenseException('Cannot deactivate license', $e->getHttpCode(), $details);
                }
                try {
                    // if new license not trial - try activate
                    if (!empty($value)) {
                        $this->logger->notice('Activate license: ' . $value);
                        $this->lexofficeInvoiceLicenseService->activateLicense($value);
                    }
                } catch (LexofficeInvoiceLicenseException $e) {
                    // if activation failed reactivate old one
                    try {
                        // if old key is no trial - try to dectivate
                        if (!empty($oldKey)) {
                            $this->logger->notice('Activate license: ' . $oldKey);
                            $this->lexofficeInvoiceLicenseService->activateLicense($oldKey);
                        }
                    } catch (LexofficeInvoiceLicenseException $e) {
                    }

                    $details = '{"action":"activate","licenseKey":"' . $value . '"}';
                    throw new LexofficeInvoiceLicenseException('Cannot deactivate license', $e->getHttpCode(), $details);
                }
            }
        }

        if (
            $key === 'lexoffice_api_authorizationcode' && !empty($value)
            && ($value !== $this->get('lexoffice_api_authorizationcode') || empty($this->get('lexoffice_api_access_token')))
        ) {
            // authorization code set -> try to get auth tokens
            $tokenInfo = $this->lexofficeService->createLexofficeAuthorizationTokens($value);
            $this->updateAuthData($tokenInfo);
        }

        if ($key === 'auth_revoke') {
            // if disabled do nothing!
            if ($value === '1') {
                // if access token available - revoke lexoffice connection 
                $accessToken = $this->get('lexoffice_api_access_token');
                if (!empty($accessToken)) {
                    $this->lexofficeService->revokeAuthentication($accessToken);
                }
                // set value to 0 to deactivate toggle again after revoke done
                $value = '0';
                // now remove all available tokens and also remove old authorization code
                $this->revokeAuthData();
            }
        }

        // Bugfix for Shop Version > 4.0 and new repository
        if ($value != null) {
            parent::set($key, $value);
        } else {
            parent::delete($key);
        }

        return $this;
    }

    /**
     * @return bool enabled
     */
    public function isEnabled()
    {
        return (bool)$this->get('is_enabled');
    }

    /**
     * @return string tax type
     */
    public function getTaxType()
    {
        return $this->get('lexoffice_tax_type');
    }

    /**
     * @param string $lxoTaxType Configuration value
     *
     * @return string tax type
     * @throws InvalidArgumentException Invalid key
     */
    public function setTaxType($lxoTaxType)
    {
        return $this->set('lexoffice_tax_type', $lxoTaxType);
    }

    /**
     * @return string small business
     */
    public function getSmallBusiness()
    {
        return $this->get('lexoffice_small_business');
    }

    /**
     * @param string $lxoSmallBusiness Configuration value
     *
     * @return string small business
     * @throws InvalidArgumentException Invalid key
     */
    public function setSmallBusiness($lxoSmallBusiness)
    {
        return $this->set('lexoffice_small_business', $lxoSmallBusiness);
    }

    /**
     * @return string license key
     */
    public function getLicenseKey()
    {
        return $this->get('license_key');
    }

    /**
     * @param string $licenseKey the license key
     *
     * @return string license key
     * @throws InvalidArgumentException Invalid key
     */
    public function setLicenseKey($licenseKey)
    {
        return $this->set('license_key', $licenseKey);
    }

    /**
     * @return string license expiration date
     */
    public function getLicenseExpirationDate()
    {
        return $this->get('license_expiration_date');
    }

    /**
     * @return string lexoffice api access token
     */
    public function getLexofficeApiAccessToken()
    {
        return $this->get('lexoffice_api_access_token');
    }

    /**
     * @param string $lexofficeApiAccessToken the lexoffice api access token
     *
     * @return string lexoffice api access token
     * @throws InvalidArgumentException Invalid key
     */
    public function setLexofficeApiAccessToken($lexofficeApiAccessToken)
    {
        return $this->set('lexoffice_api_access_token', $lexofficeApiAccessToken);
    }

    /**
     * @return string lexoffice api access token expiration timestamp
     */
    public function getLexofficeApiAccessTokenExpirationTimestamp()
    {
        return $this->get('lexoffice_api_access_token_expiration_timestamp');
    }

    /**
     * @param string $lexofficeApiAccessToken the lexoffice api access token expiration timestamp
     *
     * @return string lexoffice api access token expiration timestamp
     * @throws InvalidArgumentException Invalid key
     */
    public function setLexofficeApiAccessTokenExpirationTimestamp($lexofficeApiAccessTokenExpirationTimestamp)
    {
        return $this->set('lexoffice_api_access_token_expiration_timestamp', $lexofficeApiAccessTokenExpirationTimestamp);
    }

    /**
     * @return string lexoffice api refresh token
     */
    public function getLexofficeApiRefreshToken()
    {
        return $this->get('lexoffice_api_refresh_token');
    }

    /**
     * @param string $lexofficeApiRefreshToken the lexoffice api refresh token
     *
     * @return string lexoffice api refresh token
     * @throws InvalidArgumentException Invalid key
     */
    public function setLexofficeApiRefreshToken($lexofficeApiRefreshToken)
    {
        return $this->set('lexoffice_api_refresh_token', $lexofficeApiRefreshToken);
    }

    /**
     * @return string lexoffice api refresh token expiration timestamp
     */
    public function getLexofficeApiRefreshTokenExpirationTimestamp()
    {
        return $this->get('lexoffice_api_refresh_token_expiration_timestamp');
    }

    /**
     * @param string $lexofficeApiRefreshTokenExpirationTimestamp the lexoffice api refresh token expiration timestamp
     *
     * @return string lexoffice api refresh token expiration timestamp
     * @throws InvalidArgumentException Invalid key
     */
    public function setLexofficeApiRefreshTokenExpirationTimestamp($lexofficeApiRefreshTokenExpirationTimestamp)
    {
        return $this->set('lexoffice_api_refresh_token_expiration_timestamp', $lexofficeApiRefreshTokenExpirationTimestamp);
    }

    /**
     * @return string lexoffice contact setting
     */
    public function getLexofficeContact()
    {
        return $this->get('lexoffice_contact');
    }

    /**
     * @return string lexoffice guest contact setting
     */
    public function getLexofficeGuestContact()
    {
        return $this->get('lexoffice_guest_contact');
    }

    /**
     * @return string lexoffice footer setting
     */
    public function getLexofficeFooter()
    {
        return $this->get('lexoffice_footer');
    }

    /**
     * Check if can read and right data
     * 
     * @return bool if has a valid license and active lexoffice oauth access rights
     */
    public function hasLicensedAccessRights()
    {
        return $this->isEnabled() == true
            && $this->lexofficeInvoiceLicenseService->isLicenseExpired($this->getLicenseExpirationDate(), $this->getLicenseKey()) == false
            && !empty($this->getLexofficeApiAccessToken());
    }

    public function revokeAuthData()
    {
        //  remove all available tokens and also remove old authorization code
        parent::delete('lexoffice_api_access_token');
        parent::delete('lexoffice_api_access_token_expiration_timestamp');
        parent::delete('lexoffice_api_refresh_token');
        parent::delete('lexoffice_api_refresh_token_expiration_timestamp');
        parent::delete('lexoffice_api_authorizationcode');
    }

    public function resetAccessToken()
    {
        // remove access token and expiration to force refresh
        parent::delete('lexoffice_api_access_token');
        parent::delete('lexoffice_api_access_token_expiration_timestamp');
    }

    public function updateAuthData($tokenInfo)
    {
        $accessToken = $tokenInfo->access_token;
        $refreshToken = $tokenInfo->refresh_token;

        //calc access token expiration
        $accessTokenExpiresInSeconds = $tokenInfo->expires_in;
        $accessTokenDate = new DateTime();
        $accessTokenDate->add(new DateInterval('PT' .  $accessTokenExpiresInSeconds . 'S'));
        $accessTokenExpirationTimestamp = $accessTokenDate->getTimestamp();


        parent::set('lexoffice_api_access_token', $accessToken);
        parent::set('lexoffice_api_access_token_expiration_timestamp', $accessTokenExpirationTimestamp);
        parent::set('lexoffice_api_refresh_token', $refreshToken);

        // refresh token expiration only set once when first time receive refresh token
        if (empty($this->getLexofficeApiRefreshTokenExpirationTimestamp())) {
            // set default refresh token expiration of of 24 month
            $refreshTokenDate = new DateTime();
            $refreshTokenDate->add(new DateInterval('P24M'));
            $refreshTokenExpirationTimestamp = $refreshTokenDate->getTimestamp();
            parent::set('lexoffice_api_refresh_token_expiration_timestamp', $refreshTokenExpirationTimestamp);
        }

        return $accessToken;
    }
}
