/* --------------------------------------------------------------
   lexoffice_invoice_create_missing_documents.js 2020-09-25
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
 --------------------------------------------------------------
 */

/**
 * Create Missing Documents Modal
 *
 * This modal prompts the user for the creation of missing order documents (invoices or packing slips). Make sure
 * that you set the "orderIds", "orderIdsWithoutDocument", "type" data values to the modal before showing it.
 *
 * Example:
 *
 * ```
 * $modal
 *   .data({
 *     orderIds: [...],
 *     orderIdsWithoutDocument: [...],
 *     type: 'invoice'
 *   })
 *   .modal('show');
 * ```
 */
gxmodules.controllers.module(
  "lexoffice_invoice_create_missing_documents",
  ["modal", "loading_spinner"],
  function (data) {
    "use strict";

    // ------------------------------------------------------------------------
    // VARIABLES
    // ------------------------------------------------------------------------

    /**
     * Module Selector
     *
     * @type {jQuery}
     */
    const $this = $(this);

    /**
     * Module Instance
     *
     * @type {Object}
     */
    const module = {
      bindings: {
        createMissingDocumentsCheckbox: $this.find(
          ".create-missing-documents-checkbox"
        ),
      },
    };

    /**
     * Proceed Event Constant
     *
     * @type {String}
     */
    const PROCEED_EVENT = "create_missing_documents:proceed";

    // ------------------------------------------------------------------------
    // FUNCTIONS
    // ------------------------------------------------------------------------

    /**
     * Create missing documents.
     *
     * @param {Number[]} orderIds
     * @param {Number[]} orderIdsWithoutDocument
     * @param {String} type
     */
    function _createMissingDocuments(orderIds, orderIdsWithoutDocument, type) {
      const requests = [];
      orderIdsWithoutDocument.forEach((oID) => {
        const data = {
          type,
          oID,
          ajax: "1",
        };

        requests.push(
          $.getJSON(
            jse.core.config.get("appUrl") +
              "/admin/admin.php?do=LexofficeInvoiceModule/CreateInvoice",
            data
          ).fail(function (jqxhr, textStatus, errorThrown) {
            const title = jse.core.lang.translate(
              "TITLE_INVOICE",
              "gm_order_menu"
            );
            const message = jse.core.lang.translate(
              "MAIL_UNSUCCESS",
              "gm_send_order"
            );
            jse.libs.modal.showMessage(title, message);
            $this.modal("hide");
            return;
          })
        );
      });

      $.when(...requests).done(() => {
        const downloadPdfWindow = window.open("about:blank");
        $this.trigger(PROCEED_EVENT, [orderIds, type, downloadPdfWindow]);
      });
    }

    /**
     * Remove order IDs that do not have a document.
     *
     * @param {Number[]} orderIds
     * @param {Number[]} orderIdsWithoutDocument
     * @param {String} type
     */
    function _removeOrderIdsWithoutDocument(
      orderIds,
      orderIdsWithoutDocument,
      type
    ) {
      orderIds.forEach((id, index) => {
        if (orderIdsWithoutDocument.includes(String(id))) {
          orderIds.splice(index);
        }
      });

      if (!orderIds.length) {
        const title = jse.core.lang.translate(
          "DOWNLOAD_DOCUMENTS",
          "admin_orders"
        );
        const message = jse.core.lang.translate("NO_DOCUMENTS_ERROR", "orders");
        jse.libs.modal.showMessage(title, message);
      } else {
        $this.trigger(PROCEED_EVENT, [orderIds, type]);
      }
    }

    /**
     * On Bulk PDF Modal Confirmation Click
     *
     * Proceed with the generation of the concatenated PDF. This handler will trigger the "create_missing_document"
     */
    function _onOkButtonClick() {
      const createMissingDocuments = module.bindings.createMissingDocumentsCheckbox.get();
      const { orderIds, orderIdsWithoutDocument, type } = $this.data();

      if (createMissingDocuments) {
        _createMissingDocuments(orderIds, orderIdsWithoutDocument, type);
      } else {
        _removeOrderIdsWithoutDocument(orderIds, orderIdsWithoutDocument, type);
      }

      $this.modal("hide");
    }

    // ------------------------------------------------------------------------
    // INITIALIZE
    // ------------------------------------------------------------------------

    module.init = function (done) {
      $this.on("click", ".btn.ok", _onOkButtonClick);
      done();
    };

    return module;
  }
);
